package eu.kanade.tachiyomi.ui.library;

import android.content.Intent;
import android.content.res.Configuration;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

import com.f2prateek.rx.preferences.Preference;

import org.greenrobot.eventbus.Subscribe;
import org.greenrobot.eventbus.ThreadMode;

import java.util.ArrayList;
import java.util.List;

import butterknife.Bind;
import butterknife.ButterKnife;
import eu.davidea.flexibleadapter.FlexibleAdapter;
import eu.kanade.tachiyomi.R;
import eu.kanade.tachiyomi.data.database.models.Category;
import eu.kanade.tachiyomi.data.database.models.Manga;
import eu.kanade.tachiyomi.event.LibraryMangasEvent;
import eu.kanade.tachiyomi.ui.base.adapter.FlexibleViewHolder;
import eu.kanade.tachiyomi.ui.base.fragment.BaseFragment;
import eu.kanade.tachiyomi.ui.manga.MangaActivity;
import eu.kanade.tachiyomi.widget.AutofitRecyclerView;
import icepick.State;
import rx.Subscription;

public class LibraryCategoryFragment extends BaseFragment
        implements FlexibleViewHolder.OnListItemClickListener {

    @Bind(R.id.library_mangas) AutofitRecyclerView recycler;

    @State int position;
    private LibraryCategoryAdapter adapter;
    private List<Manga> mangas;

    private Subscription numColumnsSubscription;
    private Subscription searchSubscription;

    public static LibraryCategoryFragment newInstance(int position) {
        LibraryCategoryFragment fragment = new LibraryCategoryFragment();
        fragment.position = position;
        return fragment;
    }



    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedState) {
        // Inflate the layout for this fragment
        View view = inflater.inflate(R.layout.fragment_library_category, container, false);
        ButterKnife.bind(this, view);

        adapter = new LibraryCategoryAdapter(this);
        recycler.setHasFixedSize(true);
        recycler.setAdapter(adapter);

        if (getLibraryFragment().getActionMode() != null) {
            setMode(FlexibleAdapter.MODE_MULTI);
        }

        Preference<Integer> columnsPref = getResources().getConfiguration()
                .orientation == Configuration.ORIENTATION_PORTRAIT ?
                getLibraryPresenter().preferences.portraitColumns() :
                getLibraryPresenter().preferences.landscapeColumns();

        numColumnsSubscription = columnsPref.asObservable()
                .doOnNext(recycler::setSpanCount)
                .skip(1)
                // Set again the adapter to recalculate the covers height
                .subscribe(count -> recycler.setAdapter(adapter));

        if (savedState != null) {
            adapter.onRestoreInstanceState(savedState);

            if (adapter.getMode() == FlexibleAdapter.MODE_SINGLE) {
                adapter.clearSelection();
            }
        }

        searchSubscription = getLibraryPresenter().searchSubject
                .subscribe(text -> {
                    adapter.setSearchText(text);
                    adapter.updateDataSet();
                });



        return view;

    }

    @Override
    public void onDestroyView() {
        numColumnsSubscription.unsubscribe();
        searchSubscription.unsubscribe();
        super.onDestroyView();
    }

    @Override
    public void onResume() {
        super.onResume();
        registerForEvents();
    }

    @Override
    public void onPause() {
        unregisterForEvents();
        super.onPause();
    }

    @Override
    public void onSaveInstanceState(Bundle outState) {
        adapter.onSaveInstanceState(outState);
        super.onSaveInstanceState(outState);
    }

    @Subscribe(sticky = true, threadMode = ThreadMode.MAIN)
    public void onEvent(LibraryMangasEvent event) {
        List<Category> categories = getLibraryFragment().getAdapter().categories;
        // When a category is deleted, the index can be greater than the number of categories
        if (position >= categories.size())
            return;

        Category category = categories.get(position);
        List<Manga> mangas = event.getMangasForCategory(category);
        if (this.mangas != mangas) {
            this.mangas = mangas;
            if (mangas == null) {
                mangas = new ArrayList<>();
            }
            setMangas(mangas);
        }
    }

    protected void openManga(Manga manga) {
        getLibraryPresenter().onOpenManga(manga);
        Intent intent = MangaActivity.newIntent(getActivity(), manga);
        startActivity(intent);
    }

    public void setMangas(List<Manga> mangas) {
        if (mangas != null) {
            adapter.setItems(mangas);
        } else {
            adapter.clear();
        }
    }

    @Override
    public boolean onListItemClick(int position) {
        if (getLibraryFragment().getActionMode() != null && position != -1) {
            toggleSelection(position);
            return true;
        } else {
            openManga(adapter.getItem(position));
            return false;
        }
    }

    @Override
    public void onListItemLongClick(int position) {
        getLibraryFragment().createActionModeIfNeeded();
        toggleSelection(position);
    }

    private void toggleSelection(int position) {
        LibraryFragment f = getLibraryFragment();
        adapter.toggleSelection(position, false);
        f.getPresenter().setSelection(adapter.getItem(position), adapter.isSelected(position));

        int count = f.getPresenter().selectedMangas.size();
        if (count == 0) {
            f.destroyActionModeIfNeeded();
        }
        else {
            f.setContextTitle(count);
            f.setVisibilityOfCoverEdit(count);
            f.invalidateActionMode();
        }
    }

    public void setMode(int mode) {
        adapter.setMode(mode);
        if (mode == FlexibleAdapter.MODE_SINGLE) {
            adapter.clearSelection();
        }
    }

    private LibraryFragment getLibraryFragment() {
        return (LibraryFragment) getParentFragment();
    }

    private LibraryPresenter getLibraryPresenter() {
        return getLibraryFragment().getPresenter();
    }
}
